/* Fitxer: Persona05.java
   Descripci: Ampliaci de la classe "Persona" amb la sobreescriptura dels mtodes "equals", "hashCode"
               i "toString"
               Inclou un petit programa que comprova el funcionament dels mtodes desenvolupats.
   Autor: Isidre Guix
*/
import utilitats.AlgorismesHash;

class Persona
{
   private String dni;
   private String nom;
   private short edat;
   
   public Persona () {}
   
   public Persona (String sDni, String sNom, int nEdat)
   {
      dni = sDni;
      nom = sNom;
      if (nEdat>=0 && nEdat<=Short.MAX_VALUE)
         edat = (short)nEdat;
   }

   public Persona (Persona p)
   {
      this (p.dni, p.nom, p.edat);
   }
   
   public Persona clonar ()
   {
      return new Persona (this);
   }
   
   public int setDni(String nouDni)
   // Retorna: 0 si s'ha pogut canviar el dni
   //          1 si el nou dni no s correcte - No s'efectua el canvi
   {
      // Aqu hi podria haver una rutina de verificaci del dni
      // i actuar en conseqncia. Com que no la incorporem, retornem sempre 0
      dni = nouDni;
      return 0;
   }

   public void setNom(String nouNom)
   {
      nom = nouNom;
   }
   
   public int setEdat(int novaEdat)
   // Retorna: 0 si s'ha pogut canviar l'edat
   //          1 : Error per passar una edat negativa
   //          2 : Error per passar una edat "enorme"
   {
      if (novaEdat<0) return 1;
      if (novaEdat>Short.MAX_VALUE) return 2;
      edat = (short)novaEdat;
      return 0;
   }
   
   public String getDni() { return dni; }
   
   public String getNom() { return nom; }
   
   public short getEdat() { return edat; }
   
   public void visualitzar()
   {
      System.out.println("Dni...........:" + dni);
      System.out.println("Nom...........:" + nom);
      System.out.println("Edat..........:" + edat);
   }
   
   public final boolean equals (Object obj)
   {
      if (obj == this) return true;
      if (obj == null) return false;
      if (obj instanceof Persona) return dni.equals(((Persona)obj).dni);
      return false;
   }

   public final int hashCode ()
   {
      return AlgorismesHash.genericHash(dni.hashCode());
   }

   public String toString()
   {
      return dni + " - " + nom;
   }

   public static void main(String args[])
   {
      Persona p1 = new Persona("00000000","Pepe Gotera",33);
      Persona p2 = new Persona(p1);
      Persona p3 = new Persona("11111111","Otilio",44);
      p2.setNom ("Pepe");
      System.out.println("p1 : " + p1);
      System.out.println("p2 : " + p2);
      System.out.println("p3 : " + p3);
      System.out.println("p1 == p2 :" + (p1 == p2));
      System.out.println("p1.equals(p2) :" + p1.equals(p2));
      System.out.println("p3.equals(p2) :" + p3.equals(p2));
   }
}